@@ -21,7 +21,7 @@
 #      treated uniformly in the R code, so there seems to be no
 #      compelling need to names dirs factor.
 #      Note that this is not the same as model$terms$factors,
-#      which is treated uniformly.
+#      which is treated uniformly (but means something different).
 #
 #   5. the formal arguments to mars and earth differ, thus $call differs
 #
@@ -30,6 +30,7 @@
 #
 #   7. earth objects have a few extra components
 #
+# $$ Note that the w parameter is actually ignored by the mda:mars routines - a bug?
 #
 # Earth is much faster than mars on large models.
 # Here are some results, using the robot arm example from Friedman's Fast MARS
@@ -97,6 +98,7 @@
 # test(1000, 20,  10)
 
 #--------------------------------------------------------------------------------------------
+# $$ need to add support for multiple responses (easy but haven't had time to do it)
 
 mars.to.earth <- function(object=stop("no 'object' arg"))
 {
@@ -123,7 +125,7 @@ mars.to.earth <- function(object=stop("no 'object' arg"))
         stop1("'y' has more than one column (multiple responses are not yet supported)")
     nselected <- length(object$selected.terms)
 
-    # Fill in the [1] and [nselected] elements of rssVec and gcvVec.
+    # Fill in the [1] and [nselected] elements of rss.per.subset and gcv.per.subset.
     # This is enough for print.earth() and summary.earth() etc. to work.
     # You can fill in all the elements by calling update.earth() later.
     # We don't call update.earth() now because minor differences between pruning
@@ -133,21 +135,21 @@ mars.to.earth <- function(object=stop("no 'object' arg"))
     ntermsVec[1] = 1                                # intercept
     ntermsVec[nselected] = nselected                # nterms of selected model
 
-    rssVec <- rep(NA, length(object$all.terms))
-    rssVec[1] <- sum((y - mean(y))^2)               # null RSS
-    rssVec[nselected] <- sum(object$residuals^2)    # RSS of selected model
+    rss.per.subset <- rep(NA, length(object$all.terms))
+    rss.per.subset[1] <- sum((y - mean(y))^2)               # null RSS
+    rss.per.subset[nselected] <- sum(object$residuals^2)    # RSS of selected model
 
-    gcvVec <- get.gcv(rssVec, ntermsVec, get.nknots(ntermsVec), object$penalty, length(y))
+    gcv.per.subset <- get.gcv(rss.per.subset, ntermsVec, object$penalty, length(y))
 
-    rss <- rssVec[nselected]                        # RSS of selected model
-    gcv <- gcvVec[nselected]                        # GCV of selected model
+    rss <- rss.per.subset[nselected]                # RSS of selected model
+    gcv <- gcv.per.subset[nselected]                # GCV of selected model
 
     if(!isTRUE(all.equal(object$gcv, gcv)))         # should never happen
         warning1("the original mars GCV ", object$gcv,
             " is not equal to the re-calculated GCV ", gcv)
 
     # Renumber selected.terms, needed because we drop terms from cuts and
-    # dirs that are not in all.terms
+    # dirs that are not in all.terms (whereas mars does not)
 
     selected <- rep(NA, nrow(object$factor))
     selected[object$all.terms] <- FALSE
@@ -169,20 +171,25 @@ mars.to.earth <- function(object=stop("no 'object' arg"))
         cat("Converted to", strip.white.space(format(call)), "\n")
 
     structure(list(
-        fitted.values  = object$fitted.values,
-        residuals      = object$residuals,
-        coefficients   = object$coefficients,
-        rss            = rss,
-        rsq            = get.rsq(rss, rssVec[1]),
-        gcv            = gcv,
-        grsq           = get.rsq(gcv, gcvVec[1]),
-        bx             = object$x,
-        dirs           = object$factor[object$all.terms, , drop=FALSE],
-        cuts           = object$cuts[object$all.terms, , drop=FALSE],
-        selected.terms = selected.terms,
-        rssVec         = rssVec,
-        gcvVec         = gcvVec,
-        prune.terms    = NULL,      # init later if you want by calling update.earth()
-        call           = call),
+        bx                = object$x,
+        dirs              = object$factor[object$all.terms, , drop=FALSE],
+        cuts              = object$cuts[object$all.terms, , drop=FALSE],
+        selected.terms    = selected.terms,
+        prune.terms       = NULL, # init later if you want by calling update.earth()
+        rss               = rss,
+        rsq               = get.rsq(rss, rss.per.subset[1]),
+        gcv               = gcv,
+        grsq              = get.rsq(gcv, gcv.per.subset[1]),
+        rss.per.response  = rss,
+        rsq.per.response  = get.rsq(rss, rss.per.subset[1]),
+        gcv.per.response  = gcv,
+        grsq.per.response = get.rsq(gcv, gcv.per.subset[1]),
+        rss.per.subset    = rss.per.subset,
+        gcv.per.subset    = gcv.per.subset,
+        fitted.values     = object$fitted.values,
+        residuals         = object$residuals,
+        coefficients      = object$coefficients,
+        ppenalty          = object$ppenalty,
+        call              = call),
     class = "earth")
 }
